import { defineComponent, toRefs } from 'vue'
import styles from './index.module.less'
import { Cell, CellGroup, Radio, RadioGroup, Image, Icon } from 'vant'
import inactiveButtonIcon from './images/inactiveButtonIcon.png'
import activeButtonIcon from './images/activeButtonIcon.png'
import staffDetafult from './images/staff-default.png'
import staffActive from './images/staff-active.png'
import fixedDefault from './images/fixed-default.png'
import fixedActive from './images/fixed-active.png'
import firstDefault from './images/first-default.png'
import firstActive from './images/first-active.png'

export default defineComponent({
  name: 'staff-change',
  props: {
    radio: {
      type: String,
      default: 'staff'
    }
  },
  emits: ['change', 'close'],
  setup(props, { emit }) {
    const { radio } = toRefs(props)

    const onChangeStaff = (type: string) => {
      //
      radio.value = type
      emit('change', type)
      emit('close')
    }
    return () => (
      <div class={styles.staffContainer}>
        <div class={styles.staffTitle}>转换曲谱</div>
        <RadioGroup v-model={radio.value}>
          <CellGroup border={false}>
            <Cell
              center
              border={false}
              class={radio.value === 'staff' ? styles.active : ''}
              onClick={() => onChangeStaff('staff')}
            >
              {{
                icon: () => (
                  <Image
                    src={radio.value === 'staff' ? staffActive : staffDetafult}
                    class={styles.staffImg}
                  />
                ),
                title: () => <span class={styles.name}>五线谱</span>,
                value: () => (
                  <Radio name="staff">
                    {{
                      icon: (props: any) => (
                        <Icon
                          class={styles.boxStyle}
                          name={props.checked ? activeButtonIcon : inactiveButtonIcon}
                        />
                      )
                    }}
                  </Radio>
                )
              }}
            </Cell>
            <Cell
              center
              border={false}
              class={radio.value === 'first' ? styles.active : ''}
              onClick={() => onChangeStaff('first')}
            >
              {{
                icon: () => (
                  <Image
                    src={radio.value === 'first' ? firstActive : firstDefault}
                    class={styles.staffImg}
                  />
                ),
                title: () => <span class={styles.name}>简谱-首调</span>,
                value: () => (
                  <Radio name="first">
                    {{
                      icon: (props: any) => (
                        <Icon
                          class={styles.boxStyle}
                          name={props.checked ? activeButtonIcon : inactiveButtonIcon}
                        />
                      )
                    }}
                  </Radio>
                )
              }}
            </Cell>
            <Cell
              center
              border={false}
              class={radio.value === 'fixed' ? styles.active : ''}
              onClick={() => onChangeStaff('fixed')}
            >
              {{
                icon: () => (
                  <Image
                    src={radio.value === 'fixed' ? fixedActive : fixedDefault}
                    class={styles.staffImg}
                  />
                ),
                title: () => <span class={styles.name}>简谱-固定调</span>,
                value: () => (
                  <Radio name="fixed">
                    {{
                      icon: (props: any) => (
                        <Icon
                          class={styles.boxStyle}
                          name={props.checked ? activeButtonIcon : inactiveButtonIcon}
                        />
                      )
                    }}
                  </Radio>
                )
              }}
            </Cell>
          </CellGroup>
        </RadioGroup>
      </div>
    )
  }
})
